﻿using Syncfusion.HtmlConverter;
using Syncfusion.Pdf;
using Syncfusion.Pdf.Graphics;
using Syncfusion.Pdf.HtmlToPdf;
using Syncfusion.Pdf.Parsing;
using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ConsoleApplication9
{
    class Program
    {
        static void Main(string[] args)
        {
            var htmlStream = new StreamReader(@"c:\html_sample.html");
            var pdfName = string.Format(@"c:\pdf_sample_{0}.pdf", Guid.NewGuid());
            File.Create(pdfName).Close();

            Stream tmpResultFile = new FileStream(pdfName, FileMode.Truncate, FileAccess.ReadWrite, FileShare.None, 8192, FileOptions.None);

            // Create new Thread
            System.Threading.Thread thread = new System.Threading.Thread(() => CreatePDFStream(tmpResultFile, htmlStream.BaseStream));
            //Set the thread to STA
            thread.SetApartmentState(System.Threading.ApartmentState.STA);
            thread.Start();
            //Wait for the thread to end
            thread.Join();

            tmpResultFile.Position = 0;

        }


        private static void CreatePDFStream(Stream tmpResultFile, Stream htmlStream)
        {


            PdfDocument document = createPDFDocumentFromHTML(htmlStream);

            if (document == null) return;

            document = GetPDFDocumentWithDefaultSize(document);

            document.Save(tmpResultFile);
            document.Close(true);
        }

        private static PdfDocument GetPDFDocumentWithDefaultSize(PdfDocument loadedDocument)
        {
            MemoryStream ms = new MemoryStream();

            // Save and close the document.
            loadedDocument.Save(ms);
            loadedDocument.Close(true);

            PdfLoadedDocument lDocument = new PdfLoadedDocument(ms);
            var document = new PdfDocument();
            document.PageSettings.Margins.All = 0;

            foreach (PdfLoadedPage loadedPage in lDocument.Pages)
            {
                PdfPage p = document.Pages.Add();
                PdfTemplate template = loadedPage.CreateTemplate();
                p.Graphics.DrawPdfTemplate(template, PointF.Empty, p.Graphics.ClientSize);
            }
            //lDocument.Close(true);
            return document;
        }

        private static PdfDocument createPDFDocumentFromHTML(Stream htmlStream)
        {
            //PdfCreatorDto.CostomerLogoFile = CustomerLogoFile;
            PdfDocument document = CreatePdfDocument();
            PdfUnitConvertor convertor = new PdfUnitConvertor();
            PdfPage page = document.Pages.Add();
            float width = convertor.ConvertToPixels(page.Graphics.ClientSize.Width, PdfGraphicsUnit.Point);
            float height = convertor.ConvertToPixels(page.Graphics.ClientSize.Height, PdfGraphicsUnit.Point);

            using (HtmlConverter html = new HtmlConverter())
            {
                html.EnableJavaScript = true;
                html.AutoDetectPageBreak = true;
                // This line is comment cause syncfusion doesnt support Hyper linkes
                //html.EnableHyperlinks = true;
                string value = string.Empty;
                using (var reader = new StreamReader(htmlStream))
                {
                    value = reader.ReadToEnd();
                }

                using (HtmlToPdfResult result = html.Convert(value, "", ImageType.Metafile, (int)width, (int)height, AspectRatio.KeepWidth))
                {
                    if (result.RenderedImage == null)
                        return null;
                    PdfMetafileLayoutFormat metafileFormat = new PdfMetafileLayoutFormat();
                    metafileFormat.Break = PdfLayoutBreakType.FitPage;
                    metafileFormat.Layout = PdfLayoutType.Paginate;
                    metafileFormat.SplitImages = false;
                    metafileFormat.SplitTextLines = false;


                    // Draw metafile in PdfPage.
                    result.Render(page, metafileFormat);
                }
            }

            return document;

        }


        public static PdfDocument CreatePdfDocument()
        {
            var document = new PdfDocument();
            document.PageSettings.Width = 900;
            document.PageSettings.Height = 1250;
            document.PageSettings.SetMargins(0);
            return document;
        }



    }
}
